@@ -9,12 +9,9 @@
 
 #define WIMLIB_ADD_FLAG_GENERATE_TEST_DATA		0x08000000
 
-#define WIMLIB_CMP_FLAG_SHORT_NAMES_NOT_PRESERVED	0x00000001
-#define WIMLIB_CMP_FLAG_ATTRIBUTES_NOT_PRESERVED	0x00000002
-#define WIMLIB_CMP_FLAG_SECURITY_NOT_PRESERVED		0x00000004
-#define WIMLIB_CMP_FLAG_ADS_NOT_PRESERVED		0x00000008
-#define WIMLIB_CMP_FLAG_COMPRESSION_NOT_PRESERVED	0x00000010
-#define WIMLIB_CMP_FLAG_IMAGE2_SHOULD_HAVE_SYMLINKS	0x00000020
+#define WIMLIB_CMP_FLAG_UNIX_MODE	0x00000001
+#define WIMLIB_CMP_FLAG_NTFS_3G_MODE	0x00000002
+#define WIMLIB_CMP_FLAG_WINDOWS_MODE	0x00000004
 
 extern int
 wimlib_compare_images(WIMStruct *wim1, int image1,
